# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# JIM: do not modify on this file. I'll probably work on it.
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# METHOD 1 

lik.border <- function(llik,x.opt,x.min,x.max,r=0.10,eps=0.0001,
  opt=T,method="dicho",side){
# This function returns the point at which the value x.r
# in [x.min,x.max] at which exp(-llik(x)+llik(x.opt))=r
# In statistical terms, llik is the MINUS log lik function, x.opt the MLE,
# SIDE indicates on what side of x.opt one is going to find x.r.

# [x.min,x.max] an interval of values of x where one is likely to
# find x.r where the normed likelihood is r: only necessary for the SECANT
#  method.
# x.min is the only required value (i.e. x.max is useless) when one
# uses the STEFFENSEN method. But x.min must be in a neighbourhood
# of x.opt. This last method converges more quickly, but its is more
# sensitive to the initial condition.

# Note that x.opt is an indicating value. It first precisely computed
# using nlm (unless opt is set to F)
# eps is the precision required in the determination of x.r

# The secant method to determine the roots of an equation: 
# here, R=0.10 for example
if (opt){
 tamp <- nlm(llik,p=x.opt)
 x.opt <- tamp$estimate
 if (side==left) x.max <- min(0.99*x.opt,x.max)
 else x.min <- max(1.01*x.opt,x.max)}

lr <- log(r)
ll.opt <- llik(x.opt)

x.old <- x.min
x.new <- x.max

r.old <- r.min <- -llik(x.old)+ll.opt
r.new <- r.max <- -llik(x.new)+ll.opt

dx <- x.max-x.min

ret <- switch(method,
 secant={
  if (dx==0){
   cat("x.min and x.max must be different","\n")
   return(1)}
  while (abs(dx)>eps){
   x.new <- x.min-(r.min-lr)*(dx/(r.max-r.min))
   r.new <- -llik(x.new)+ll.opt
   if (r.new>lr) {
    if (side=="left"){
     x.max <- x.new
     r.max <- r.new}
    else {
     x.min <- x.new
     r.min <- r.new}}
   else {
    if (side=="left"){
     x.min <- x.new
     r.min <- r.new}
    else {
    x.max <- x.new
    r.max <- r.new}}
   dx <- x.max-x.min
   cat(x.new,exp(r.new),"\n")}},
 Steffensen={
  x.new <- x.old
  r.new <- r.old
  cat("starting value:", x.new,"\n")
  while (abs(r.new-lr)>eps){
    tamp <- x.new-((r.new-lr)^2/(-llik(x.new+r.new-lr)+ll.opt-r.new))
    x.new <- tamp
    cat("Xnew: ",x.new,"\n")
    r.new <- -llik(x.new)+ll.opt}},
 dicho={
  if ((r.min-lr)*(r.max-lr)>0) {
    cat("[x.min;x.max] does not contain the required R value","\n")
#    cat("R at x.min",exp(r.min),"\n")
#    cat("R at x.max",exp(r.max),"\n")
    return(1)}
  while(dx>eps) {
   x.new <- 0.5*(x.min+x.max)
   r.new <- -llik(x.new)+ll.opt
#   cat("(",x.min,x.new,x.max,")",exp(r.new),"\n")
   if (r.new>lr) {
    if (side=="left")
     x.max <- x.new
    else x.min <- x.new}
   else {
    if (side=="left")
     x.min <- x.new
    else x.max <- x.new}
   dx <- x.max-x.min}})

return(list(x.opt=x.opt,x.r=x.new,r=exp(r.new)))}



# Application of the secant method to obtain the 10% lik interval for y
#  under the distributions fitted with the first line below.

y <- c(296,296,300,302,300,304,303,299,293,294,294,293,295,287,288,297,305,
307,307,304,303,304,304,309,309,309,307,306,304,300,296,301,298,295,
295,293,292,297,294,293,306,303,301,303,308,305,302,301,297,299)

ret <- (y[2:50]-y[1:49])/y[1:49]


# Beta regressed on time
z.opt <- stablelm(y = ret, Delta = 1/y[1:49], gamm = ret ~ 1, delta = ret ~ 1,
   beta = ret ~ tim, alpha = ret ~ 1, igamm = 0.0318538, idelta = -0.2729406,
   ibeta = c(2.6405736,-20.6216196), ialpha = -0.5393566,
   hessian=F,output=F,integration="Press")

#Log likelihood               128.2899 
#No. of obs                   49 
#No. of estimated parameters  5 
#No. of parameters            5 
#Degrees of freedom           44 
#Optimized AIC                133.2899 
#AIC                          133.2899 
#
#Coefficients:
#    estimate  se
#1    0.03424   1
#2   -0.26920   1
#3    2.64072   1
#4  -20.62196   1
#5   -0.53849   1

# f <- function(y) {
#  tamp <- stable(y=y,gam=z.opt$gamma[1],delta=z.opt$delta[1],beta=z.opt$beta[1],alpha=z.opt$alpha[1])
# return(-log(tamp))}

# y.start <- 0
# tamp <- nlm(f,p=y.start)
# x.opt <- tamp$estimate
# y1 <- lik.border(f,x.opt=x.opt,x.min=-0.02,x.max=0,r=0.10,opt=F)

n <- length(ret)
y.opt <- rep(0,n)
#y.r10 <- rep(0,n)
y.r90 <- rep(0,n)
#r.r10 <- rep(0,n)
r.r90 <- rep(0,n)

for (i in 1:n){
 f <- function(y) {
  tamp <- stable(y=y,gam=z.opt$gamma[i],delta=z.opt$delta[i],
                     beta=z.opt$beta[i],alpha=z.opt$alpha[i],integration="Press")
 return(-log(tamp))}
 y.start <- z.opt$gamma[i]
 tamp <- nlm(f,p=y.start,hessian=F)
 x.opt <- tamp$estimate
 y.opt[i] <- x.opt
# y1 <- lik.border(f,x.opt=x.opt,x.min=-0.05,x.max=0.9*x.opt,r=0.10,
#                   opt=F,method="dicho",eps=0.0001,side="left")
# y.r10[i] <- y1$x.r
# r.r10[i] <- y1$r
 y2 <- lik.border(f,x.opt=x.opt,x.min=1.1*x.opt,x.max=0.1,r=0.10,
                   opt=F,method="dicho",eps=0.0001,side="right")
 y.r90[i] <- y2$x.r
 r.r90[i] <- y2$r
 cat(i,") ","[",y.r10[i],";",y.r90[i],"] ",y.opt[i],"\n")
}

#postscript(file="fit2.eps",landscape=FALSE,width=2,height=5.7)
x <- idx-1
plot(x,ret,xlab="Day",ylab="Share return",pch=4,type="n",xlim=c(0,50))
out.idx <- which(abs(ret)>0.016)
not.out.idx <- which((!abs(ret)>0.016))
points(x[not.out.idx],ret[not.out.idx])
points(x[out.idx],ret[out.idx],pch="0")
lines(x,y.opt,lty="solid")
lines(x,y.r10,lty="dotted")
lines(x,y.r90,lty="dotted")

#> y.opt
# [1]  0.0075661690  0.0073521662  0.0070957238  0.0067923177  0.0064354603
# [6]  0.0060180903  0.0055360235  0.0049856540  0.0043645823  0.0036761778
#[11]  0.0029264151  0.0021262670  0.0012915569  0.0004400229 -0.0004080368
#[16] -0.0012311209 -0.0020137914 -0.0027401445 -0.0034026794 -0.0039961758
#[21] -0.0045200379 -0.0049767220 -0.0053693257 -0.0057053018 -0.0059887763
#[26] -0.0062285551 -0.0064278785 -0.0065945359 -0.0067326272 -0.0068468597
#[31] -0.0069410377 -0.0070185035 -0.0070821002 -0.0071342282 -0.0071774258
#[36] -0.0072117877 -0.0072402915 -0.0072643328 -0.0072825439 -0.0072980247
#[41] -0.0073106430 -0.0073209244 -0.0073295284 -0.0073366035 -0.0073416776
#[46] -0.0073462002 -0.0073498822 -0.0073528793 -0.0073560979
#> y.r10
# [1] -0.01399473 -0.01433796 -0.01481372 -0.01542250 -0.01605629 -0.01671776
# [7] -0.01740788 -0.01812718 -0.01887547 -0.01964952 -0.02023930 -0.02084927
#[13] -0.02112122 -0.02155381 -0.02159687 -0.02144784 -0.02129450 -0.02094121
#[19] -0.02057233 -0.02018216 -0.01976744 -0.01932670 -0.01885905 -0.01836651
#[25] -0.01819788 -0.01783147 -0.01761603 -0.01738163 -0.01730199 -0.01703601
#[31] -0.01692883 -0.01698153 -0.01685438 -0.01671979 -0.01674948 -0.01660314
#[37] -0.01662283 -0.01663944 -0.01665202 -0.01666272 -0.01650182 -0.01650896
#[43] -0.01651494 -0.01651985 -0.01652337 -0.01652651 -0.01652907 -0.01653115
#[49] -0.01653338
#> y.r90
# [1] 0.01897668 0.01912767 0.01941967 0.01985084 0.02023676 0.02057220
# [7] 0.02103825 0.02145222 0.02200007 0.02231671 0.02259415 0.02284391
#[13] 0.02269613 0.02254470 0.02221101 0.02190577 0.02124241 0.02057649
#[19] 0.01981197 0.01910346 0.01834938 0.01764963 0.01700218 0.01640119
#[25] 0.01584363 0.01542669 0.01504403 0.01468909 0.01435849 0.01415328
#[31] 0.01396547 0.01379216 0.01363091 0.01358496 0.01344152 0.01341120
#[37] 0.01328061 0.01325937 0.01324328 0.01322960 0.01311294 0.01310385
#[43] 0.01309624 0.01308998 0.01308549 0.01308149 0.01307823 0.01307558
#[49] 0.01307273
#> r.r10
# [1] 0.09957964 0.10036005 0.10039514 0.09972061 0.09947478 0.09955452
# [7] 0.09982202 0.10007908 0.10006402 0.09948009 0.10011667 0.09948109
#[13] 0.10106021 0.09915071 0.09974632 0.10083429 0.10020594 0.10076671
#[19] 0.10038232 0.09947610 0.09856073 0.09822884 0.09919080 0.10226268
#[25] 0.09752442 0.10000985 0.09914003 0.10049888 0.09696697 0.10280230
#[31] 0.10285064 0.09590070 0.09854587 0.10259809 0.09839256 0.10447580
#[37] 0.10156683 0.09914402 0.09721156 0.09560295 0.10433566 0.10323851
#[43] 0.10233257 0.10159027 0.10101392 0.10052733 0.10013072 0.09980757
#[49] 0.09951133
#> r.r90
# [1] 0.09704854 0.10069597 0.10093906 0.09871302 0.09907048 0.10122939
# [7] 0.10089170 0.10153656 0.09953008 0.10032948 0.10029214 0.09906113
#[13] 0.10086573 0.10071825 0.10079805 0.09902516 0.09975941 0.09960204
#[19] 0.09992658 0.09950854 0.09969222 0.09969360 0.09969655 0.09986124
#[25] 0.10026679 0.09986818 0.09976618 0.09998422 0.10050371 0.10013964
#[31] 0.10001491 0.10010119 0.10037172 0.09961925 0.10018156 0.09966215
#[37] 0.10042560 0.10008458 0.09979165 0.09955861 0.10056081 0.10040400
#[43] 0.10027842 0.10017673 0.10008697 0.10001777 0.09996142 0.09991553
#[49] 0.09988597


# METHOD 2

# Computation of the fitted values (y.hat) and of 10% lik intervals
# for the model described in paper2.out

#> ret
#  [1]  0.000000000  0.013333333  0.006622517 -0.006666667  0.013157895
#  [6] -0.003300330 -0.013377926 -0.020477816  0.003401361  0.000000000
# [11] -0.003412969  0.006779661 -0.027874564  0.003472222  0.030303030
# [16]  0.026229508  0.006514658  0.000000000 -0.009868421 -0.003300330
# [21]  0.003289474  0.000000000  0.016181230  0.000000000  0.000000000
# [26] -0.006514658 -0.003267974 -0.006578947 -0.013333333 -0.013513514
# [31]  0.016611296 -0.010067114 -0.010169492  0.000000000 -0.006825939
# [36] -0.003424658  0.016835017 -0.010204082 -0.003412969  0.042483660
# [41] -0.009900990 -0.006644518  0.006600660  0.016233766 -0.009836066
# [46] -0.009933775 -0.003322259 -0.013468013  0.006688963

#> tim
#  [1] 0.00 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.11 0.12 0.13 0.14
# [16] 0.15 0.16 0.17 0.18 0.19 0.20 0.21 0.22 0.23 0.24 0.25 0.26 0.27 0.28 0.29
# [31] 0.30 0.31 0.32 0.33 0.34 0.35 0.36 0.37 0.38 0.39 0.40 0.41 0.42 0.43 0.44
# [46] 0.45 0.46 0.47 0.48

z.opt <- stablelm(y = ret, Delta = 1/y[1:49], gamm = ret ~ tim, delta = ret ~ 1, 	beta = ret ~ 1, alpha = ret ~ 1, igamm = c(0.79, -1.9267), idelta = -0.1642	, ibeta = -2.0563, ialpha = -0.8525,integration="Press")
# list returned by stablelm after optimization 

y <- seq(-0.03,0.03,by=0.001)
n <- length(ret)
ny <- length(y)

y.opt <- rep(0,n)
y.r10 <- rep(0,n)
y.r90 <- rep(0,n)
r.r10 <- rep(0,n)
r.r90 <- rep(0,n)

# The procedure below must be used if delta or beta or alpha do vary with
# gamma. Otherwise, we know that the distance(mode,gamma) is a constant
#(ie. if gamma grows by some amount, then so does the mode.
#
#idx <- 1:n
#for (i in 1:n){
# zz <- stable(y=y,gam=z.opt$gamma[i],delta=z.opt$delta[i],beta=z.opt$beta[i],alpha=z.opt$alpha[i])
# relative.lik.y <- zz/max(zz)
# idx.opt <- min(which(relative.lik.y==1))
# y.opt[i] <- y[idx.opt]
# y.r10[i] <- y[max(1,min(which(relative.lik.y>0.10)-1))]
# y.r90[i] <- y[min(ny,max(which(relative.lik.y>0.10)))]
# r.r10[i] <- relative.lik.y[max(1,min(which(relative.lik.y>0.10)-1))]
# r.r90[i] <- relative.lik.y[min(ny,max(which(relative.lik.y>0.10)))]
# cat(i,"  (",y.r10[i],y.opt[i],y.r90[i],")","\n")
#}

# The procedure below can be used to determine the mode of the stable distribution 
# for different values of gamma because, when delta, beta and alpha do NOT vary
# with  gamma, then distance(mode,gamma) is a constant.
#
zz <- stable2(y=y,gam=z.opt$gamma[1],delta=z.opt$delta[1],beta=z.opt$beta[1],alpha=z.opt$alpha[1],integration="Simpson")
relative.lik.y <- zz/max(zz)
idx.opt <- min(which(relative.lik.y==1))
y.opt[1] <- y[idx.opt]
dd <- y.opt[1]-z.opt$gamma[1] # This is distance(mode,gamma)
y.opt <- z.opt$gamma+dd

y.r10[1] <- y[max(1,min(which(relative.lik.y>0.10)-1))]
dd <- y.opt[1]-y.r10[1] # (Left) width of the 10% lik interval 
y.r10 <- y.opt-dd

y.r90[1] <- y[min(ny,max(which(relative.lik.y>0.10)))]
dd <- y.r90[1]-y.opt[1] # (Right) width of the 10% lik interval 
y.r90 <- y.opt+dd
r.r10 <- relative.lik.y[max(1,min(which(relative.lik.y>0.10)-1))]
r.r90 <- relative.lik.y[min(ny,max(which(relative.lik.y>0.10)))]

print(lm(y.opt~x)) # provides the corresponding regression for the mode

#Call:
#lm(formula = y.opt ~ x)
#
#Coefficients:
#  (Intercept)             x 
# 0.0001000000 -0.0001783758 

#postscript(file="fit.eps",landscape=FALSE,width=2,height=5.7)
zn <- lm(ret~x)
x <- idx-1
plot(x,ret,xlab="Day",ylab="Share return",pch=4,type="n",xlim=c(0,50))
out.idx <- which(abs(ret)>0.016)
not.out.idx <- which((!abs(ret)>0.016))
points(x[not.out.idx],ret[not.out.idx])
lines(x,z8$gamma,lty="dashed") # Fitted gamma
#lines(x,zn$fitted,lty="dotdashed") # Fitted normal
lines(x,zn$fitted,lty="25") # Fitted normal
lines(x,zn$fitted,lty="dotted") # Fitted normal
lines(c(x[1],x[n]),c(y.r10[1],y.r10[n]),lty="dotted")
lines(c(x[1],x[n]),c(y.r90[1],y.r90[n]),lty="dotted")
lines(c(x[1],x[n]),c(y.opt[1],y.opt[n]),lty="solid")
points(x[out.idx],ret[out.idx],pch="0")
# arrows(x[40],zn$fitted[40],x[49]+0.2,zn$fitted[49]+0.01)


x <- idx-1
plot(x,ret,xlab="Day",ylab="Share return",bty="l",pch=4,type="n",xlim=c(0,50))
out.idx <- which(abs(ret)>0.016)
not.out.idx <- which((!abs(ret)>0.016))
points(x[not.out.idx],ret[not.out.idx])
lines(x[1:37],y.opt[1:37],lty="solid")
lines(x[1:37],y.r10[1:37],lty="dotted")
lines(x[1:37],y.r90[1:37],lty="dotted")





