#############################################################################
# ******************** DIFFERENCE WITH profile.r :  *************************
# Share value corresponding to the last Monday of August 1991
# (26/08/91) removed from Buckle (1995) dataset (cf. Bank holiday in England)
# In Buckle, it was just set equal to the last closing value of the share,
# i.e. the one recorded on Friday 23/08/91 ==> not a real datum.
#############################################################################


#### CAREFUL ######
# Computation of the profile likelihood were done before parameters
# were systematically transformed before use. Thus, do not 
# forget to modify <delta=delta[i]> to <delta=delta.g(delta[i])>
# to compute the profile lik at the requested <...> !!

library(stable)

y <- c(296,296,300,302,300,304,303,299,293,294,294,293,295,287,288,297,305,
307,304,303,304,304,309,309,309,307,306,304,300,296,301,298,295,
295,293,292,297,294,293,306,303,301,303,308,305,302,301,297,299)

ret <- (y[2:49]-y[1:48])/y[1:48]

# starting values for igamma, idisp, ibeta, ialpha
ig <- 0.15554
id <- -0.17242
ib <- -2.24593
ia <-  0.07271

maxlik <- stablereg(y=ret,delta = 1/y[1:48],
         loc= ret ~ 1, disp = ret ~ 1, skew = ret ~ 1, tail = ret~1,
         iloc = ig, idisp = id, iskew = ib, itail = ia, hessian=F)
dev.opt <- maxlik$llik
gamm.hat <- maxlik$gamma[1]
delta.hat <- maxlik$delta[1]
beta.hat <- maxlik$beta[1]
alpha.hat <- maxlik$alpha[1]

ig <- maxlik$coef[1]
id <- maxlik$coef[2]
ib <- maxlik$coef[3]
ia <- maxlik$coef[4]


# Computation of a the relative profile lik for the
# parameter gamm in the stationary stable model with the Buckle data set.

gamm <- c(seq(-0.005,0.000,by=0.001),seq(0.0005,0.0040,by=0.0005),seq(0.005,0.010,by=0.001))
gamm.relative.lik <- 0*gamm

for (i in 1:length(gamm)){
 tamp <- stablereg(y=ret,delta = 1/y[1:48],
         loc= gamm[i], disp = ret ~ 1, skew = ret ~ 1, tail = ret~1,
         oloc=F, idisp = id,iskew = ib, itail = ia, hessian=F,
         integration="Simpson",output=F)
 ddev <- tamp$llik-dev.opt  # dev.opt is the -llik at gamm.hat
                           # ddev is the difference in -llik
 gamm.relative.lik[i] <- exp(dev.opt-tamp$llik)
 cat(i,") loc=",gamm[i],"  R=",gamm.relative.lik[i],"\n")}

gamm <- 
  c(-0.0050,-0.0040,-0.0030,-0.0020,-0.0010,0.0000,0.0010,0.0015,
     0.001728456,0.0020,0.0025, 0.0030, 0.0035, 0.0040, 0.0050,
     0.0060,0.0070,0.0080, 0.0090, 0.0100)
gamm.relative.lik <-
c(0.01926549,0.04693083,0.1247842,0.2994763,0.568665,0.8299199,0.9185456, 0.9729106,
0.9970178,0.997295,0.9801826,0.951401,0.9155429,0.8760166,0.7948167,0.7188146,
0.6523626,0.596064,0.5486065,0.5035373) 

plot(gamm,gamm.relative.lik,type="l",xlab=expression(gamma),ylab="Normed profile likelihood")

# Computation of a the relative profile lik for the
# parameter delta in the stationary stable model with the Buckle data set.

delta <- c(seq(0.004,0.00751,by=0.0005),0.007789586,0.0080,0.0085,seq(0.009,0.018,by=0.001))
delta.relative.lik <- 0*delta

for (i in 1:length(delta)){
 tamp <- stablereg(y=ret,delta = 1/y[1:48],
         loc= ret ~ 1, disp = disp.g(delta[i]), skew = ret ~ 1, tail = ret~1,
         odisp=F, iloc = ig, iskew = ib, itail = ia, hessian=F, integration="Press")
 delta.relative.lik[i] <- exp(dev.opt-tamp$llik)
 cat(i,") disp=",delta[i],"  R=",delta.relative.lik[i],"\n")}

delta <-
  c(0.004,0.0045,0.005,0.0055,0.006,0.0065,0.007,0.0075,0.007789586,0.008,
  0.0085,0.009,0.01,0.011,0.012,0.013,0.014,0.015,0.016,0.017,0.018)  

delta.relative.lik <-
c(0.00036256,0.00330632,0.01119598,0.08908968,0.251216,0.5057754,0.7716605,
0.951358,0.9946497,0.9992486,0.952012,0.8814313,0.7524138,0.6500502,
0.574583,0.5161793,0.4497341,0.4318206,0.4011829,0.3753898,0.2521345) 

plot(delta[-c(16,17)],delta.relative.lik[-c(16,17)],type="l",
     xlab=expression(delta),ylab="Normed profile likelihood",
     xlim=c(0.005,0.016))
plot(delta,delta.relative.lik,type="l",xlab=expression(delta),ylab="Normed profile likelihood")

# Computation of a the relative profile lik for the
# parameter beta in the stationary stable model with the Buckle data set.

beta <- c(seq(-1,-0.69,by=0.01),seq(-0.7,0.5,by=0.1))
beta.relative.lik <- 0*beta

for (i in 1:length(beta)){
 tamp <- stablereg(y=ret, delta = 1/y[1:48],
         loc= ret ~ 1, disp = ret ~ 1, skew = skew.g(beta[i]), tail = ret~1,
         oskew = F, iloc = ig, idisp = id, itail = ia, hessian=F,
         integration="Press",output=F)
 beta.relative.lik[i] <- exp(dev.opt-tamp$llik)
 cat(i,") skew=",beta[i],"  R=",beta.relative.lik[i],"\n")}

beta <- c(seq(-1,-0.69,by=0.01),seq(-0.7,0.5,by=0.1))
beta.relative.lik <-
c(0.9497314,0.9510394, 0.9524888, 0.9540936, 0.95587  , 0.9578364, 0.9600143,
0.9624291, 0.96511  , 0.9680914, 0.971412 , 0.9751601, 0.979287 , 0.983827 , 
0.9886454, 0.993311 , 0.9971057, 0.9993751, 0.9999632, 0.9986833, 0.9956892, 
0.9925605, 0.986476 , 0.9792102, 0.9707708, 0.9599402, 0.9494325, 0.9383017, 
0.9264376, 0.9139751, 0.9009983, 0.8875833, 0.9009983, 0.7565644, 0.6112829, 
0.4819869, 0.37317  , 0.2844164, 0.2136123, 0.1582425, 0.1156804, 0.08359535,
0.0600574, 0.04357978,0.03271886)
plot(beta,beta.relative.lik,type="l",xlab=expression(beta),xlim=c(-1,0.3),
     ylab="Normed profile likelihood")

# Computation of a the relative profile lik for the
# parameter alpha in the stationary stable model with the Buckle data set.

alpha <- c(seq(1.05,1.51,by=0.05),1.518169,seq(1.55,2.01,by=0.05))
alpha.relative.lik <- 0*alpha

for (i in 1:length(alpha)){
 tamp <- stablereg(y=ret,delta = 1/y[1:48],
         loc= ret ~ 1, disp = ret ~ 1, skew = ret ~ 1, tail = tail.g(alpha[i]),
         otail=F, iloc = ig, idisp = id, iskew=ib, hessian=F, integration="Simpson")
 alpha.relative.lik[i] <- exp(dev.opt-tamp$llik)
 cat(i,") tail=",alpha[i],"  R=",alpha.relative.lik[i],"\n")}

alpha <- c(seq(1.05,1.51,by=0.05),1.527547,seq(1.55,2.01,by=0.05))

alpha <- c(seq(1.05,1.51,by=0.05),1.518169,seq(1.55,2.01,by=0.05))
alpha.relative.lik <- 
c(0.02215155,0.1447086,0.3408205,0.4927944,0.6100912,0.7293556,0.835605,0.9179837,
0.971564, 0.9965418,1.,       0.99795,  0.9839894,0.9639516,0.9460687,0.8772913,
0.7314042,0.5358434,0.3287379,0.1492791,0.03312315)
plot(alpha,alpha.relative.lik,type="l",xlab=expression(alpha),ylab="Normed profile likelihood")


# Final plot
# Note that I had to move beta' and delta' to the left by hand
#   on the abscissa label
postscript(file="profileb.eps",horizontal=FALSE,width=6,height=6)
par(mfrow=c(2,2),mar=c(5,3,2,2))
plot(alpha,alpha.relative.lik,type="l",xlab=expression(alpha),ylab="")
plot(beta,beta.relative.lik,type="l",xlab=expression(paste(beta,"'")),ylab="",xlim=c(-1,0.3))
plot(gamm,gamm.relative.lik,type="l",xlab=expression(gamma),ylab="")
plot(delta[-c(16,17)],delta.relative.lik[-c(16,17)],type="l",
     xlab=expression(paste(delta,"'")),ylab="", xlim=c(0.005,0.016))
dev.off()

